@@ -6,7 +6,9 @@ import base64
 import json
 from datetime import datetime
 from uuid import UUID
+import uuid
 from google.protobuf.json_format import MessageToJson
+from google.protobuf.duration_pb2 import Duration
 import os
 
 from indykite_sdk.utils.hash_methods import encrypt_bcrypt, encrypt_sha256
@@ -14,7 +16,7 @@ from indykite_sdk.identity import IdentityClient
 from indykite_sdk.config import ConfigClient
 from indykite_sdk.authorization import AuthorizationClient
 from indykite_sdk.indykite.config.v1beta1.model_pb2 import (SendGridProviderConfig, MailJetProviderConfig, AmazonSESProviderConfig, MailgunProviderConfig)
-from indykite_sdk.indykite.config.v1beta1.model_pb2 import (EmailServiceConfig, AuthFlowConfig, OAuth2ClientConfig, IngestMappingConfig)
+from indykite_sdk.indykite.config.v1beta1.model_pb2 import (EmailServiceConfig, AuthFlowConfig, OAuth2ClientConfig, IngestMappingConfig, WebAuthnProviderConfig)
 from indykite_sdk.indykite.config.v1beta1.model_pb2 import OAuth2ProviderConfig, OAuth2ApplicationConfig
 from indykite_sdk.indykite.identity.v1beta2.import_pb2 import ImportDigitalTwinsRequest, ImportDigitalTwin
 from indykite_sdk.indykite.identity.v1beta2.import_pb2 import PasswordCredential, PasswordHash, Bcrypt, SHA256
@@ -397,6 +399,20 @@ Property ID and value of the property where the value is a reference
     update_ingest_mapping_config_node_parser.add_argument("display_name", help="Display name")
     update_ingest_mapping_config_node_parser.add_argument("description", help="Description")
 
+    # create_webauthn_provider_config_node
+    create_webauthn_provider_config_node_parser = subparsers.add_parser("create_webauthn_provider_config_node")
+    create_webauthn_provider_config_node_parser.add_argument("app_space_id", help="AppSpace (gid)")
+    create_webauthn_provider_config_node_parser.add_argument("name", help="Name (not display name)")
+    create_webauthn_provider_config_node_parser.add_argument("display_name", help="Display name")
+    create_webauthn_provider_config_node_parser.add_argument("description", help="Description")
+
+    # update_webauthn_provider_config_node
+    update_webauthn_provider_config_node_parser = subparsers.add_parser("update_webauthn_provider_config_node")
+    update_webauthn_provider_config_node_parser.add_argument("config_node_id", help="Config node id (gid)")
+    update_webauthn_provider_config_node_parser.add_argument("etag", help="Etag")
+    update_webauthn_provider_config_node_parser.add_argument("display_name", help="Display name")
+    update_webauthn_provider_config_node_parser.add_argument("description", help="Description")
+
     # read_oauth2_provider
     read_oauth2_provider_parser = subparsers.add_parser("read_oauth2_provider")
     read_oauth2_provider_parser.add_argument("oauth2_provider_id", help="Oauth2 provider id (gid)")
@@ -493,6 +509,32 @@ Property ID and value of the property where the value is a reference
     start_forgotten_password.add_argument("digital_twin_id", help="gid ID of the digital twin with forgotten password")
     start_forgotten_password.add_argument("tenant_id", help="gid ID of the tenant")
 
+    # create_email_invitation
+    create_email_invitation = subparsers.add_parser("create_email_invitation")
+    create_email_invitation.add_argument("tenant_id", help="gid ID of the tenant")
+    create_email_invitation.add_argument("email", help="invitee's email")
+
+    # create_email_invitation_with_date
+    create_email_invitation_with_date = subparsers.add_parser("create_email_invitation_with_date")
+    create_email_invitation_with_date.add_argument("tenant_id", help="gid ID of the tenant")
+    create_email_invitation_with_date.add_argument("email", help="invitee's email")
+
+    # create_mobile_invitation
+    create_mobile_invitation = subparsers.add_parser("create_mobile_invitation")
+    create_mobile_invitation.add_argument("tenant_id", help="gid ID of the tenant")
+    create_mobile_invitation.add_argument("mobile", help="invitee's mobile")
+
+    # check_invitation_state
+    check_invitation_state = subparsers.add_parser("check_invitation_state")
+    check_invitation_state.add_argument("reference_id", help="external ID of the invitation reference")
+
+    # resend_invitation
+    resend_invitation = subparsers.add_parser("resend_invitation")
+    resend_invitation.add_argument("reference_id", help="external ID of the invitation reference")
+
+    # cancel_invitation
+    cancel_invitation = subparsers.add_parser("cancel_invitation")
+    cancel_invitation.add_argument("reference_id", help="external ID of the invitation reference")
 
     args = parser.parse_args()
     local = args.local
@@ -1129,13 +1171,9 @@ Property ID and value of the property where the value is a reference
 
     elif command == "read_config_node":
         config_node_id = args.config_node_id
-        config_node = client_config.read_config_node(config_node_id,[])
+        config_node = client_config.read_config_node(config_node_id, [])
         if config_node:
             print_response(config_node)
-            if config_node.auth_flow_config:
-                source = config_node.auth_flow_config.source
-                if source:
-                    print(json.loads(source.decode('utf-8')))
         else:
             print("Invalid config node id")
 
@@ -1368,6 +1406,60 @@ Property ID and value of the property where the value is a reference
             print("Invalid update ingest mapping config node response")
         return update_ingest_mapping_config_node_response
 
+    elif command == "create_webauthn_provider_config_node":
+        location = args.app_space_id
+        name = args.name
+        display_name = args.display_name
+        description = args.description
+
+        webauthn_provider_config = WebAuthnProviderConfig(
+            relying_parties={"http://localhost":"localhost"},
+            attestation_preference="CONVEYANCE_PREFERENCE_NONE",
+            authenticator_attachment="AUTHENTICATOR_ATTACHMENT_DEFAULT",
+            require_resident_key=False,
+            user_verification="USER_VERIFICATION_REQUIREMENT_PREFERRED",
+            registration_timeout=Duration(seconds=30),
+            authentication_timeout=Duration(seconds=60)
+        )
+
+        create_webauthn_provider_config_node_response = client_config.create_webauthn_provider_config_node(
+            location, name, display_name, description, webauthn_provider_config, [])
+        if create_webauthn_provider_config_node_response:
+            print_response(create_webauthn_provider_config_node_response)
+        else:
+            print("Invalid create webauthn provider config node response")
+        return create_webauthn_provider_config_node_response
+
+    elif command == "update_webauthn_provider_config_node":
+        config_node_id = args.config_node_id
+        etag = args.etag
+        display_name = args.display_name
+        description = args.description
+
+        webauthn_provider_config = WebAuthnProviderConfig(
+            relying_parties={"http://localhost":"localhost"},
+            attestation_preference="CONVEYANCE_PREFERENCE_INDIRECT",
+            authenticator_attachment="AUTHENTICATOR_ATTACHMENT_DEFAULT",
+            require_resident_key=False,
+            user_verification="USER_VERIFICATION_REQUIREMENT_REQUIRED",
+            registration_timeout=Duration(seconds=30),
+            authentication_timeout=Duration(seconds=60)
+        )
+
+        update_webauthn_provider_config_node_response = client_config.update_webauthn_provider_config_node(
+            config_node_id,
+            etag,
+            display_name,
+            description,
+            webauthn_provider_config,
+            [])
+
+        if update_webauthn_provider_config_node_response:
+            print_response(update_webauthn_provider_config_node_response)
+        else:
+            print("Invalid update webauthn provider config node response")
+        return update_webauthn_provider_config_node_response
+
     elif command == "read_oauth2_provider":
         oauth2_provider_id = args.oauth2_provider_id
         config = client_config.read_oauth2_provider(oauth2_provider_id, [])
@@ -1741,6 +1833,62 @@ Property ID and value of the property where the value is a reference
         else:
             print("Invalid forgotten password response")
 
+    elif command == "create_email_invitation":
+        reference_id = str(uuid.uuid4())
+        # any unique external reference id
+        print(reference_id)
+        tenant_id = args.tenant_id
+        email = args.email
+        invitation_response = client.create_email_invitation(tenant_id, reference_id, email, invite_at_time=None,
+                                                             expire_time=None, message_attributes=None)
+        if invitation_response is not None:
+            print(invitation_response)
+        else:
+            print("Invalid invitation response")
+
+    elif command == "create_email_invitation_with_date":
+        reference_id = str(uuid.uuid4())
+        print(reference_id)
+        tenant_id = args.tenant_id
+        email = args.email
+        t = datetime.now().timestamp()
+        invite_at_time_in_seconds = int(t) + 3600
+        expire_time_in_seconds = invite_at_time_in_seconds + 172800 # now + 2 days example
+        message_attributes = {"attr1": "value1"}
+        invitation_response = client.create_email_invitation(tenant_id, reference_id, email,
+                                                             invite_at_time_in_seconds,
+                                                             expire_time_in_seconds,
+                                                             message_attributes)
+        if invitation_response is not None:
+            print(invitation_response)
+        else:
+            print("Invalid invitation response")
+
+    elif command == "check_invitation_state":
+        reference_id = args.reference_id
+        # check with either reference_id or invitation_token
+        invitation_response = client.check_invitation_state(reference_id, None)
+        if invitation_response is not None:
+            print_response(invitation_response)
+        else:
+            print("Invalid invitation response")
+
+    elif command == "resend_invitation":
+        reference_id = args.reference_id
+        invitation_response = client.resend_invitation(reference_id)
+        if invitation_response is not None:
+            print(invitation_response)
+        else:
+            print("Invalid invitation response")
+
+    elif command == "cancel_invitation":
+        reference_id = args.reference_id
+        invitation_response = client.cancel_invitation(reference_id)
+        if invitation_response is not None:
+            print(invitation_response)
+        else:
+            print("Invalid invitation response")
+
 
 def print_verify_info(digital_twin_info):  # pragma: no cover
     print("Digital twin info")
